#include <stdio.h>
#include <iostream>
#include <string>

#include <lear/cmdline.h>

#include "rawdescio.h"

using namespace std;
using namespace lear;

#define SQ(x) ((x)*(x))

#define SHOW_STAT_RAW(n,st) \
  { \
  bool const differ = (r1.st!=r2.st); \
  anyDiffer |= differ; \
  if (verbose > 1) { \
  cout << setw(15) << n << "  " \
       << setw(10) << r1.st << "  " << setw(10) << r2.st << "  " \
       << setw( 7) << (differ?"** Y **":"n") << endl; \
  } \
  }
#define SHOW_STAT(st) SHOW_STAT_RAW(#st, st())

void compute(string const &f1, string const &f2, int verbose) {
  RawDescIn r1(f1, verbose);
  RawDescIn r2(f2, verbose);

  // Are the files compatible with each other?
  bool anyDiffer = false;
  if (verbose > 1) {
    cout << "File 1: " << f1 << endl;
    cout << "File 2: " << f2 << endl;
    cout << endl;
    cout << "   Statistic       File 1      File 2    Differ?" << endl;
    cout << "---------------  ----------  ----------  -------" << endl;
  }
  SHOW_STAT(featureCount);
  SHOW_STAT(featureLength);
  SHOW_STAT(descNumber);
  SHOW_STAT_RAW("WinSizeX", extent()[0]);
  SHOW_STAT_RAW("WinSizeY", extent()[1]);
  if (verbose > 1) cout << endl;
  if (anyDiffer) {
    ThrowException("Descriptor geometries and/or number of descriptors "
                   "differs for \"" << f1 << "\" and \"" << f2 << "\".");
  }

  // Look at the data
  Array1DType feature1; feature1.resize(r1.featureLength());
  Array1DType feature2; feature2.resize(r2.featureLength());
  double sdist  = 0.f;
  double ssdist = 0.f;
  double sm1    = 0.f;
  double ssm1   = 0.f;
  double sm2    = 0.f;
  double ssm2   = 0.f;
  //int       featureLen = r1.featureLength();
  ptrdiff_t nFeatures  = r1.featureCount();
  for (ptrdiff_t i=0; i<nFeatures; i++) {
    r1.next(feature1);
    r2.next(feature2);
    double dist = sqrt(blitz::sum(SQ(feature1 - feature2)));
    sdist  += dist;
    ssdist += SQ(dist);
    double m1  = sqrt(blitz::sum(SQ(feature1))); 
    sm1  += m1;
    ssm1 += SQ(m1);
    double m2  = sqrt(blitz::sum(SQ(feature2)));
    sm2  += m2;
    ssm2 += SQ(m2);
  }

  if (verbose > 0) {
    cout.flush();
    printf("%38s: % 1.4f (stdev=% 1.4f)\n", 
           "mean window-wise L2 magnitude, file 1",
           sm1 / nFeatures, sqrt((ssm1/(nFeatures-1)) - SQ(sm1/nFeatures)));
    printf("%38s: % 1.4f (stdev=% 1.4f)\n", 
           "mean window-wise L2 magnitude, file 2",
           sm2 / nFeatures, sqrt((ssm2/(nFeatures-1)) - SQ(sm2/nFeatures)));
    printf("%38s: % 1.4f (stdev=% 1.4f)\n", 
           "mean window-wise L2 distance",
           sdist / nFeatures, 
           sqrt((ssdist/(nFeatures-1)) - SQ(sdist/nFeatures)));
    fflush(stdout);
  }
}

int main(int argc, char **argv) {
  int    verbose;
  string f1, f2;

  lear::Cmdline cmdline;
  cmdline.commandName("compare_rhog");
  cmdline.version("0.0.2", "Author: Gerald Dalley "
                  "(mailto: dalleyg@ieee.org)");
  cmdline.brief("Compare two .RHOG descriptor files");
  cmdline.description("Compare two .RHOG descriptor files.  If they have "
                      "the same size of features and the same number of "
                      "features, the actual feature data is compared too.");
  cmdline.addOption()
    ("verbose,v",option<int>(&verbose)
     ->defaultValue(2)->minValue(0)->maxValue(9),
     "verbose level")
    ;
  cmdline.addArgument()
    ("file1", option<std::string>(&f1), "first file to compare")
    ("file2", option<std::string>(&f2), "second file to compare")
    ;
  int status = cmdline.parse(argc, argv);
  if (status != cmdline.ok) return status;
  
  try {
    compute(f1, f2, verbose);
  } catch(lear::Exception& e) {
    cerr << "Caught " << e.what() << endl;
    return 1;
  } catch(std::exception& e) {
    cerr << "Caught " << (e.what() ? e.what() : "UNKNOWN ERROR") << endl;
    return 1;
  } catch(...) {
    cerr << "Caught unknown exception" << endl;
    return 1;
  }
  return 0;
}

/*
   reset; make -j16 && (((time nice -n +10 app/dump_rhog --cuda=0 --samples 0 --infile /scratch/dalleyg/dalalTriggs/trunk/experiments/inria_20080211/w64x128c8x8_all/hard_neg.txt --outfile cpuhard.RHOG --poscases 2416 --negcases 12180 --memorylimit 1700 --dumphard 1 --hardscore 0 --verbose=9 --imageext=png --window=64,128 --winstride=8,8 --topleft=16,16 --cellsize=8,8 --numcell=2,2 --orientbin=9 --descstride=8,8 --proc=rgb_sqrt --norm=l2hys --fullcirc=0 --wtscale=2 --gscale=0 --maxvalue=0.2 --epsilon=1 ) 2>&1) > cpuhard.log) && (((time nice -n +10 app/dump_rhog --cuda=1 --samples 0 --infile /scratch/dalleyg/dalalTriggs/trunk/experiments/inria_20080211/w64x128c8x8_all/hard_neg.txt --outfile gpuhard.RHOG --poscases 2416 --negcases 12180 --memorylimit 1700 --dumphard 1 --hardscore 0 --verbose=9 --imageext=png --window=64,128 --winstride=8,8 --topleft=16,16 --cellsize=8,8 --numcell=2,2 --orientbin=9 --descstride=8,8 --proc=rgb_sqrt --norm=l2hys --fullcirc=0 --wtscale=2 --gscale=0 --maxvalue=0.2 --epsilon=1 ) 2>&1) > gpuhard.log) && app/compare_rhog cpuhard.RHOG gpuhard.RHOG
   reset; make -j16 && (((time nice -n +10 app/dump_rhog --cuda=1 --samples 0 --infile /scratch/dalleyg/dalalTriggs/trunk/experiments/inria_20080211/w64x128c8x8_all/hard_neg.txt --outfile gpuhard.RHOG --poscases 2416 --negcases 12180 --memorylimit 1700 --dumphard 1 --hardscore 0 --verbose=9 --imageext=png --window=64,128 --winstride=8,8 --topleft=16,16 --cellsize=8,8 --numcell=2,2 --orientbin=9 --descstride=8,8 --proc=rgb_sqrt --norm=l2hys --fullcirc=0 --wtscale=2 --gscale=0 --maxvalue=0.2 --epsilon=1 ) 2>&1) > gpuhard.log) && app/compare_rhog cpuhard.RHOG gpuhard.RHOG
   reset; make -j16 && app/compare_rhog cpuhard.RHOG gpuhard.RHOG
 */
