#ifndef _LEAR_HISTOGRAM_PROCESSOR_H_
#define _LEAR_HISTOGRAM_PROCESSOR_H_

#include <lear/cvision/dhistogram.h>

namespace lear {
  // Histogram accumulator.  Writes out histogram upon destruction.
  struct HistProcessor {
    typedef float                               RealType;
    HistProcessor(
      std::string const &histfile_,
      RealType const minTh_ = -20,
      RealType const extTh_ = 40, 
      RealType const bandwidth_ = 0.01);

    virtual ~HistProcessor();

    lear::DiscreteHistogram<int,RealType>::HistType const &histogram() const {
      return histall_.data();
    }

    virtual void operator()(RealType const r);

    virtual std::string toString() { 
      return "Histogram Processor";
    }

  protected:
    /// output file
    std::string const histfile_;

    bool const dohistout_;
    /// min, max threshold and bandwidth for computing score histogram
    RealType const minTh_, maxTh_;

    /// score histogram, can be used to compute ROC curves
    lear::DiscreteHistogram<int,RealType> histall_;
  };
}
#endif //_LEAR_HISTOGRAM_PROCESSOR_H_
