#ifndef _LEAR_MASK_PROCESSRESULT_H_
#define _LEAR_MASK_PROCESSRESULT_H_

#include <lear/classifier/ppresult.h>
#include <lear/image/imageio.h>
#include <string>
#include <lear/cvision/iprocessor.h>

namespace lear {
  // Takes an input mask file.  Any detections that land on black
  // pixels are filtered out.
  struct Mask_ProcessResult : public ProcessResult {
    typedef ProcessResult            Parent;
    typedef IProcessor::GrayImage    GrayImage;   // Array<float,2>
    typedef IProcessor::RGBImage     RGBImage;  
    typedef blitz::TinyVector<int,2> IndexType;

    Mask_ProcessResult(std::string const &maskfile, 
                       IndexType   const &initialExt/*=IndexType(-1,1)*/) {
      lear::ImageIO::read(maskfile, mask);
      xscale = (initialExt[0] == -1) ? 1 : mask.extent()[0] / double(initialExt[0]);
      yscale = (initialExt[1] == -1) ? 1 : mask.extent()[1] / double(initialExt[1]);
    }

    virtual bool operator()(DetectInfo const &r) {
      // TODO: allow the user to select different rules for checking
      // the mask.  Some good options might be: check all pixels in
      // the top row of the detector window, check a pixel
      // corresponding to the head tip, check all pixels within the
      // window, check all pixels within the avsize portion of the
      // window, check the centroid, etc.

      // for now, check the top center of the window
      int const mx = int((r.lbound[0]+(r.extent[0]/2.0f)) * xscale); // center
      int const my = int((r.lbound[1]                   ) * yscale); // top
      if (mx < 0 || mx >= mask.extent()[0]) return false;
      if (my < 0 || my >= mask.extent()[1]) return false;
      return mask(mx,my);
    }
    
    virtual std::string toString() {
      return "Detection masker process result";
    }

  private:
    GrayImage mask;
    double xscale, yscale;
  };
  
};

#endif
