#ifndef _LEAR_MS_PROCESS_RESULT_H_
#define _LEAR_MS_PROCESS_RESULT_H_

#include <list>
#include <string>
#include <blitz/tinyvec.h>
#include <lear/cvision/transfunc.h>
#include <lear/classifier/th_processresult.h>

namespace lear {

  /*
   * Runs meanshift and non-maxima suppression on the detector results.
   */
  struct MeanShift_ProcessResult : public Threshold_ProcessResult {
    typedef Threshold_ProcessResult                    Parent;

    typedef Parent::RealType                    RealType;
    typedef Parent::DetectionWin                DetectionWin;
    typedef DetectionWin::const_iterator        const_iterator;
    typedef blitz::TinyVector<RealType,3>       PointType;
    typedef blitz::TinyVector<RealType,2>       SigmoidType;
    typedef blitz::TinyVector<int,2>            IndexType;

    MeanShift_ProcessResult(IndexType const extent_,
                            RealType const threshold_,
                            RealType const finalthreshold_,
                            SigmoidType const score2prob_,
                            PointType const sigma_,
                            int const transfunc,
                            int const verbose=0);

    ~MeanShift_ProcessResult() { delete sigmoid; }

    /** Do non-maximum suppression on the detection results */
    virtual void doit();
    virtual std::string toString() { 
      return std::string("Mean Shift NonMax Process Result\n    ")
        + sigmoid->toString();
    }

  protected:
    IndexType           const extent_;
    RealType            const finalthreshold_;
    PointType           const sigma_;
    TransFunc<RealType>      *sigmoid;
    int                 const verbose;
  };

}
#endif // _LEAR_MS_PROCESS_RESULT_H_
