#include <iomanip>
#include <iostream>
#include <lear/exception.h>
#include <lear/util/fileutil.h>
#include <lear/cvision/iprocessor.h>
#include <lear/image/imageio.h>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
namespace fs=boost::filesystem;

#include <lear/classifier/drawbestthresh.h>

using namespace std;

lear::DrawBestThreshold::DrawBestThreshold(
  std::string const outmat_,
  bool        const outfileIsDir_,
  std::string const &tag_)
  : PostProcessResult(), outmat_(outmat_), outfileIsDir_(outfileIsDir_),
  tag_(tag_)
{}

bool lear::DrawBestThreshold::verifyImageOut(std::string const outmat_,
                                             bool const outfileIsDir_) 
{
  bool doMatOut_ = (!outmat_.empty());
  // check if outmat is valid
  if (doMatOut_ && outfileIsDir_) {
    fs::path outdir(outmat_);
    if (fs::exists(outdir)) {
      // make sure that outfile is also a directory
      if (!fs::is_directory(outdir)) {
        cerr<< "WARNING:: 'infile' is a directory/list, "
          " thus 'outmat'=" << outmat_ 
          <<"' must also be a directory."
          << "Disabling image output." << endl;
        doMatOut_ = false;
      }
    } else {
      // it does not exist, create a directory
      fs::create_directory(outdir);
      if (!fs::exists(outdir)) {
        cerr<< "WARNING:: Unable to create output image directory "
          << outmat_ << "Disabling image output." << endl;
        doMatOut_ = false;
      } else {
        cerr<< "WARNING:: Directory " << outdir.string() << 
          " does not exist. Creating it." << endl;
      }
    }
  }
  return doMatOut_;
}

void lear::DrawBestThreshold::write(const_iterator s, const_iterator e, 
                                    lear::PostProcessResult::RGBImage const &inimage,
                                    std::string                       const &infile)
{
  std::string fname;
  if (outfileIsDir_) {
    fname = outmat_ + "/" + lear::basename(infile) + "." + tag_ + ".csv";
  } else {
    fname = lear::basename(outmat_) + "." + tag_ + ".csv";
  }

  // Create "out" with all nans
  int const w = inimage.extent()[0];
  int const h = inimage.extent()[1];
  float *out = new float[w*h];
  {
    float *o = out;
    for (int i=0; i<w*h; i++) {
      *o++ = -std::numeric_limits<float>::quiet_NaN();
    } 
  }

  // Paint bounding boxes with their score
  for (; s!=e; ++s) {
    float const score = s->score;
    int const xmin = max(0, s->lbound[0]);
    int const xmax = min(w, s->lbound[0] + s->extent[0]);
    int const ymin = max(0, s->lbound[1]);
    int const ymax = min(h, s->lbound[1] + s->extent[1]);
    for (int y=ymin; y<ymax; ++y) {
      float *o = out + y*w + xmin;
      for (int x=xmin; x<xmax; ++x, ++o) {
        if (!(*o >= score)) *o = score;
      }
    }
  }

  // Save output
  ofstream f(fname.c_str());
  {
    float const *o = out;
    for (int y=0; y<h; y++) {
      for (int x=0; x<w; x++) {
#ifdef WIN32
        if (_isnan(*o)) {
#else
        if (isnan(*o)) {
#endif
          f << "NaN,";
        } else {
          f << *o << ',';
        }
        o++;
      }
      f << endl;
    }
  }

  delete [] out;
}

