#include <lear/classifier/tracker_ppresult.h>
#include <vector>

using namespace std;

void lear::Tracker_PPResult::write(const_iterator s, const_iterator e,
                                   lear::PostProcessResult::RGBImage const &inimage,
                                   std::string                       const &filename)
{
  // Run the tracker
  vector<ConnectedComponent> cc(1024); // manually manage resizes to avoid copy constructor costs
  int nCC=0;
  for (; s!=e; ++s, ++nCC) {
    if (size_t(nCC+1) >= cc.size()) { cc.resize(cc.size()*2); } 
    ConnectedComponent &c = cc[nCC];  // all units are in terms of pixels
    c.id          = nCC;
    c.area        = blitz::product(s->extent);
    c.rect.x      = s->lbound[0];
    c.rect.y      = s->lbound[1];
    c.rect.width  = s->extent[0];
    c.rect.height = s->extent[1];
    c.cx          = s->lbound[0] + s->extent[0]/2;
    c.cy          = s->lbound[1] + s->extent[1]/2;
  }
  int64 const timestamp = int64(frame * 1e3 / fps); // milliseconds
  if (tracker->processFrame(NULL, NULL, &cc[0], nCC, timestamp) != Trk_OK) {
    ThrowException("Tracker error." << endl
                   << "           frame: " << frame << endl
                   << "      detections: " << nCC << " (max allowed is " << tracker->getMaxCC() << ")" << endl
                   << "          tracks: " << tracker->getNumObjects() << " (max allowed is " << tracker->getMaxTrk() << ")" << endl
                   );
  }

  // Write results (auto-converting units to normalized frame-based
  // sizes)
  int const w = (frameExtent[0]==-1) ? inimage.extent()[0] : frameExtent[0];
  int const h = (frameExtent[1]==-1) ? inimage.extent()[1] : frameExtent[1];
  double        const  xscale = 1. / w;
  double        const  yscale = 1. / h;
  int           const  nObjs = tracker->getNumObjects();
  TrackerObject const *objs  = tracker->getTrackedObjects();
  for (int o=0; o<nObjs; o++) {
    TrkFormatBits format = HasTimeAndId; 
    if (objs[o].type == TrackerObject::FirstInstance || 
        objs[o].type == TrackerObject::NormalInstance) {
      format = (TrkFormatBits)(format | HasObjStats);
    }
    TrkRecord rec(format, timestamp, (uint32)frame,
                  (uint64)objs[o].id, objs[o], NULL, xscale, yscale);
    if (trkoutput->write(rec) != Trk_OK) {
      ThrowException("Track writing error");
    }
  }

  frame++;
}

