#include <lear/windowsystem.h>
#include <lear/image/imageio.h>
#include <cctype>
#include <lear/util/fileutil.h>
#include <lear/blitz/ext/tvmutil.h>

using namespace lear;

#ifdef WIN32
using namespace ATL;
ATL::CImage lear::ImageIO::currImage;
#endif

std::string ImageIO::imageFormat(std::string const &imagename) {
#ifndef WIN32
  Imlib_Image image = imlib_load_image(imagename.c_str());

  if (image)// if the load was successful
  {   // set the image we loaded as the current context image to work on
    imlib_context_set_image(image);

    std::string format (imlib_image_format());
    imlib_free_image();
    return format;
  } else {
    return std::string();
  }
#else
  currImage.Destroy();
  return lear::extension(imagename);
#endif
}


blitz::TinyVector<int,2> ImageIO::size(std::string const &imagename) {
#ifndef WIN32
  Imlib_Image image = imlib_load_image(imagename.c_str());;

  if (image)// if the load was successful
  {   // set the image we loaded as the current context image to work on
    imlib_context_set_image(image);
    int w = imlib_image_get_width(), 
      h = imlib_image_get_height();
    imlib_free_image();
    return blitz::TinyVector<int,2> (w,h);
  } else {
    ThrowException("Unable to read image: " + imagename);
  }
#else
  currImage.Destroy();
  CString n(imagename.c_str());
  HRESULT hr = currImage.Load(n);
  if (SUCCEEDED(hr)) {
    return blitz::TinyVector<int,2>(currImage.GetWidth(), currImage.GetHeight());
  } else {
    ThrowException("Unable to read image: " + imagename);
  }
#endif
}

void ImageIO::save(std::string const &imagename){
#ifndef WIN32
  std::string ext = lear::extension(imagename);
  //int n = imagename.rfind(".");
  //std::string ext = imagename.substr(n+1,std::string::npos); 
  if (ext.length())
    imlib_image_set_format(ext.c_str());
  else
    imlib_image_set_format("png");

  /* save the image */
  Imlib_Load_Error reterr;
  imlib_save_image_with_error_return(imagename.c_str(), &reterr);
  imlib_free_image();

  if (reterr != IMLIB_LOAD_ERROR_NONE) {
    ThrowException(std::string("Unable to save file: ") + imagename);
  } 
#else
  CString n(imagename.c_str());
  HRESULT hr = currImage.Save(n);
  currImage.Destroy();
  if (!SUCCEEDED(hr)) {
    ThrowException("Unable to save file: " + imagename);
  }
#endif
}
