# Configure the ffmpeg library
# Gerald Dalley <dalleyg@ieee.org>, March 2009

# Quality disclaimer: this file was written with minimal experience with 
# autoconf and m4 scripts.  Although it has been written in a manner that
# seems mostly correct and logical to the author, an experienced individual 
# could probably clean it up substantially.


# AC_FFMPEG
# ---------
# Verifies that the ffmpeg libraries have been installed and are usable.  
AC_DEFUN([AC_FFMPEG],
  [
    # Make sure the basic headers are available
    _AC_FFMPEG_AVCODEC_H
    _AC_FFMPEG_AVFORMAT_H
    # Check for libraries that might be required, in reverse linking order
    _AC_FFMPEG_OPTIONAL_LIB([bzip2],   [bz2],                  [BZ2_decompress])
    _AC_FFMPEG_OPTIONAL_LIB([avutil],  [avutil_pic avutil],    [av_log])
    _AC_FFMPEG_OPTIONAL_LIB([swscale], [swscale_pic swscale],  [sws_scale])
    _AC_FFMPEG_REQUIRED_LIB([avcodec], [avcodec_pic avcodec],  [avcodec_decode_video])
    _AC_FFMPEG_REQUIRED_LIB([avformat],[avformat_pic avformat],[av_open_input_file])
    _AC_FFMPEG_OPTIONAL_LIB([avdevice],[avdevice_pic avdevice],[avdevice_register_all])
    _AC_FFMPEG_OPTIONAL_LIB([postproc],[postproc_pic postproc],[pp_postprocess])
    _AC_FFMPEG_OPTIONAL_LIB([avfilter],[avfilter_pic avfilter],[avfilter_register])
    # Report results
    have_ffmpeg=yes
    AC_DEFINE(HAVE_FFMPEG,,[ffmpeg video processing libraries])
    AC_SUBST(FFMPEGLIBS)
  ])

# _AC_FFMPEG_AVCODEC_H
# --------------------
# Looks for the avcodec.h header file.  If it's found in libavcodec/avcodec.h, 
# then LIBAVCODEC_INCLUDE_DIR is defined.  If it's found in ffmpeg/avcodec.h,
# FFMPEG_INCLUDE_DIR is defined.
AC_DEFUN([_AC_FFMPEG_AVCODEC_H],
  [
    AC_CHECK_HEADERS(
      [libavcodec/avcodec.h],
      [
        AC_DEFINE(LIBAVCODEC_INCLUDE_DIR,,["libavcodec headers are in libavcodec/"])
        have_avcodec="yes"
      ],
      [
        AC_CHECK_HEADERS([ffmpeg/avcodec.h],
          [
            AC_DEFINE(FFMPEG_INCLUDE_DIR,,["libavcodec headers are in ffmpeg/"])
            have_avcodec="yes"
          ]
        )
      ]
    )
    AS_IF([test x"$have_avcodec" != "xyes"],
      [
        echo "*** Could not find avcodec.h, a required ffmpeg header."
        exit 1
      ]
    )
  ]
)

# _AC_FFMPEG_AVFORMAT_H
# --------------------
# Looks for the avformat.h header file.  If it's found in libavformat/avformat.h, 
# then LIBAVFORMAT_INCLUDE_DIR is defined.  If it's found in ffmpeg/avformat.h,
# FFMPEG_INCLUDE_DIR is defined.
AC_DEFUN([_AC_FFMPEG_AVFORMAT_H],
  [
    AC_CHECK_HEADERS(
      [libavformat/avformat.h],
      [
        AC_DEFINE(LIBAVFORMAT_INCLUDE_DIR,,["libavformat headers are in libavformat/"])
        have_avformat="yes"
      ],
      [
        AC_CHECK_HEADERS([ffmpeg/avformat.h],
          [
            AC_DEFINE(FFMPEG_INCLUDE_DIR,,["libavformat headers are in ffmpeg/"])
            have_avformat="yes"
          ]
        )
      ]
    )
    AS_IF([test x"$have_avformat" != "xyes"],
      [
        echo "*** Could not find avformat.h, a required ffmpeg header."
        exit 1
      ]
    )
  ]
)

# _AC_FFMPEG_OPTIONAL_LIB([name],[libs],[function])
# ------------------------------------------------
# Tries to find the requested libraries.  If any are found, link argments are 
# prepended to FFMPEGLIBS.
AC_DEFUN([_AC_FFMPEG_OPTIONAL_LIB],
  [
    AC_SEARCH_LIBS([$3],[$2],
      [
        AS_VAR_SET([have_$1], [yes])
        AC_DEFINE(AS_TR_CPP([HAVE_$1]),[],[$1])
        FFMPEGLIBS="-l$ac_lib $FFMPEGLIBS"
        break
      ],
      [
        have_$1=no
      ],[$FFMPEGLIBS]
    )
  ]
)

# _AC_FFMPEG_REQUIRED_LIB([name],[libs],[function])
# ------------------------------------------------
# Same as _AC_FFMPEG_OPTIONAL_LIB, but not finding one of the libs is fatal.
AC_DEFUN([_AC_FFMPEG_REQUIRED_LIB],
  [
    _AC_FFMPEG_OPTIONAL_LIB($1,$2,$3)
    AS_IF([test x"$have_$1" == "xyes"],
          [],
          [echo "Could not find library: '$1'"
           exit 1])
  ]
)
