#ifndef PDCUDA_IMTRANSFER_H
#define PDCUDA_IMTRANSFER_H

#include <cuda_runtime.h>
#include <cufft.h>

namespace pdcuda
{
  // Contraints on 3D texture sizes, as of compute capability 1.3
  // (c.f. NVidia CUDA Programming Guide).
  static int const MAX_3D_TEXTURE_EXT = 2048; // max size for any dimension
  bool valid3dRgbArraySize(int w, int h);


  float4 *rgbToDeviceRgba(float const *data, int w, int h,
                          bool dataOnHost, bool dataColumnMajor);

  cudaArray *rgbToCudaArray2d(float const *data, int w, int h,
                               bool dataOnHost, bool dataColumnMajor);

  cudaArray *rgbaToCudaArray2d(float4 const *data, int w, int h,
                               bool dataOnHost, bool dataColumnMajor);

  cudaArray *rgbToCudaArray3d(float const *data, int w, int h,
                              bool dataOnHost, bool dataColumnMajor);


#if 0 // To be implemented later, if needed
  // FFT transfers not yet implemented

  // 3x planar 2D ffts
  // retval[c*ow*h + y*ow + x], ow=(w/2+1)
  cufftComplex *rgbToFft2d(float const *data, int w, int h,
                           bool dataOnHost, bool dataColumnMajor);

  // 1x 3D fft
  // retval[y*w*od + x*od + c], od=(3/2+1)=2
  cufftComplex *rgbToFft3d(float const *data, int w, int h,
                           bool dataOnHost, bool dataColumnMajor);
#endif

};

#endif
