#ifndef PDCUDA_MATH_CUH
#define PDCUDA_MATH_CUH

// Some convenience mathematical operators for CUDA data structures.
// This header file should only be included in CUDA (.cu) files, not
// in strict C/C++ source files.

// Adapted from:
//  http://forums.nvidia.com/index.php?showtopic=28289

//-----------------------------------------------------------------------------
// Addition
__device__ __host__ inline float4
operator+(float4 const a, float4 const b)
{
  return make_float4(a.x+b.x, a.y+b.y, a.z+b.z, a.w+b.w);
}

__device__ __host__ inline float4&
operator+=(float4 &a, float4 const b)
{
  a.x += b.x; 
  a.y += b.y; 
  a.z += b.z; 
  a.w += b.w; 
  return a;
}

//-----------------------------------------------------------------------------
// Subtraction
__device__ __host__ inline float4
operator-(float4 const a, float4 const b)
{
  return make_float4(a.x-b.x, a.y-b.y, a.z-b.z, a.w-b.w);
}

//-----------------------------------------------------------------------------
// Multiplication
__device__ __host__ inline float4
operator*(float4 const a, float4 const b)
{ 
  return make_float4(a.x*b.x, a.y*b.y, a.z*b.z, a.w*b.w); 
}

__device__ __host__ inline float4
operator*(float const a, float4 const b)
{ 
  return make_float4(a*b.x, a*b.y, a*b.z, a*b.w); 
}

__device__ __host__ inline float4
operator*(float4 const a, float const b)
{ 
  return make_float4(a.x*b, a.y*b, a.z*b, a.w*b); 
}

//-----------------------------------------------------------------------------
// Division
__device__ __host__ inline float4
operator/(float4 const a, float4 const b)
{
  return make_float4(a.x/b.x, a.y/b.y, a.z/b.z, a.w/b.w);
}

__device__ __host__ inline float4
operator/(float4 const a, float const b)
{
  return make_float4(a.x/b, a.y/b, a.z/b, a.w/b);
}

__device__ __host__ inline float4
operator/(float const a, float4 const b)
{
  return make_float4(a/b.x, a/b.y, a/b.z, a/b.w);
}

__device__ __host__ inline float4&
operator/=(float4 &a, float4 const b)
{
  a.x /= b.x; 
  a.y /= b.y; 
  a.z /= b.z; 
  a.w /= b.w; 
  return a;
}

//-----------------------------------------------------------------------------
// sqrt
__device__ __host__ inline float4
sqrtf(float4 const x) {
  return make_float4(sqrtf(x.x), sqrtf(x.y), sqrtf(x.z), sqrtf(x.w));
}

#endif
