function H = borderedRect(coords, borderthickness, colors)
%borderedRect(coords, borderthickness, colors)
%  coords: [x y w h]
%  borderthickness: non-negative scalar
%  colors: 4x3 matrix where each row is an RGB value.  
%          Rows are for N, S, E, W colors, where N is the min Y value (like
%          axis image)

x = coords(1);
y = coords(2);
w = coords(3);
h = coords(4);
t = borderthickness;

wasHold = ishold;

H = {};

if t>0
  % north
  H{end+1} = patch([x x+w x+w+t x-t], [y y y-t y-t], colors(1,:), 'LineStyle','none'); 
  hold on;
  % south
  H{end+1} = patch([x x+w x+w+t x-t], [y+h y+h y+h+t y+h+t], colors(2,:), 'LineStyle','none');
  % east
  H{end+1} = patch([x+w x+w x+w+t x+w+t], [y y+h y+h+t y-t], colors(3,:), 'LineStyle','none');
  % west
  H{end+1} = patch([x x x-t x-t], [y y+h y+h+t y-t], colors(4,:), 'LineStyle','none');
  
  colors(:) = 0; % black outlines
end

H{end+1} = line([x x+w], [y y], 'Color',colors(1,:)); % north
hold on;
H{end+1} = line([x+w x+w], [y y+h], 'Color',colors(3,:)); % east
H{end+1} = line([x x+w], [y+h y+h], 'Color',colors(2,:)); % south
H{end+1} = line([x x], [y y+h], 'Color',colors(4,:)); % west

H{end+1} = line([x-t x+w+t], [y-t y-t], 'Color',colors(1,:)); % north
H{end+1} = line([x+w+t x+w+t], [y-t y+h+t], 'Color',colors(3,:)); % east
H{end+1} = line([x-t x+w+t], [y+h+t y+h+t], 'Color',colors(2,:)); % south
H{end+1} = line([x-t x-t], [y-t y+h+t], 'Color',colors(4,:)); % west

H = cell2mat(fullcollapse(H));

if ~wasHold
  hold off;
end
