function drawAnnotation(head,bbox, drawhip,thickness,color, taggedColor,taggedParts)
%drawAnnotation(headcenter,bbox, drawhip,thickness,color, taggedColor,taggedParts)
%  Draws a pedestrian annotation on the current image.
%

if nargin<3,  drawhip     = 1;       end
if nargin<4,  thickness   = 2;       end
if nargin<5,  color       = [0 1 0]; end
if nargin<6,  taggedColor = [0 0 1]; end
if nargin<7,  taggedParts = 0;       end

HEADCENTER =   1;
VERTEX_NW  =   2;
VERTEX_NE  =   4;
VERTEX_SE  =   8;
VERTEX_SW  =  16;
EDGE_N     =  32;
EDGE_E     =  64;
EDGE_S     = 128;
EDGE_W     = 256;
VERTICES  = VERTEX_NW   + VERTEX_NE + VERTEX_SE + VERTEX_SW;
EDGES     = EDGE_N      + EDGE_E    + EDGE_S    + EDGE_W;
PARTS     = HEADCENTER  + VERTICES  + EDGES;

% bounding box
borderedRect([bbox(1) bbox(2) bbox(3)-bbox(1) bbox(4)-bbox(2)], ...
  thickness, ...
  [colorPicker(color,taggedParts,EDGE_N);...
  colorPicker(color,taggedParts,EDGE_S);...
  colorPicker(color,taggedParts,EDGE_E);...
  colorPicker(color,taggedParts,EDGE_W)]);

wasHold = ishold;
hold on;

% draw hip line for selected
if drawhip
  hipY = mean(bbox([2 4]));
  line(bbox([1 3]), [hipY hipY], 'LineStyle',':', 'Color',color);
end

% head
plot(head(1),head(2),'+','MarkerSize',7,'LineWidth',1,'Color',colorPicker(color,taggedParts,HEADCENTER));
headHalfHeight = (head(2) - bbox(2));
headHalfWidth  = headHalfHeight * 0.75;
ellipse(head, [headHalfWidth headHalfHeight], 17, ':','Color', colorPicker(color,taggedParts,HEADCENTER));
line([-headHalfWidth headHalfWidth]+head(1), head(2)+headHalfHeight+[0 0], 'LineStyle',':', 'Color',colorPicker(color,taggedParts,HEADCENTER));

% corners
if bitand(VERTEX_NW,taggedParts), plot(bbox(1),bbox(2), '.', 'MarkerSize',5, 'Color',taggedColor); end
if bitand(VERTEX_NE,taggedParts), plot(bbox(3),bbox(2), '.', 'MarkerSize',5, 'Color',taggedColor); end
if bitand(VERTEX_SW,taggedParts), plot(bbox(1),bbox(4), '.', 'MarkerSize',5, 'Color',taggedColor); end
if bitand(VERTEX_SE,taggedParts), plot(bbox(3),bbox(4), '.', 'MarkerSize',5, 'Color',taggedColor); end

if ~wasHold,
  hold off;
end

%-----------------------------------------
  function c = colorPicker(untaggedColor,taggedParts,part)
    if bitand(part,taggedParts)
      c = taggedColor;
    else
      c = untaggedColor;
    end
  end
end