%% exptExploreHeightDistr
% After annotating the 25 of the Grand Central 1159407741750 frames, I
% wondered what the distribution of annotation heights is.  If many of them
% are smaller than 128, then we might need to train a low-res detector.


%% Store README info

F = fopen(todaysResultsDir('README.heights.txt'), 'w');
fprintf(F,'height* created by %s\n', which(mfilename));
fclose(F);

%% Get all known height data

ymaxes  = {};
heights = {};
names   = {};

load(inriaDir('train/inria/annotations_20080211.mat'));
flat = flattenAnnotations(annotationsHier);
ymaxes{end+1}  = flat.bboxes(4,:);
heights{end+1} = flat.bboxes(4,:) - flat.bboxes(2,:);
names{end+1}   = 'inria 20080211 train';

load(inriaDir('test/inria/annotations_20080211.mat'));
flat = flattenAnnotations(annotationsHier);
ymaxes{end+1}  = flat.bboxes(4,:);
heights{end+1} = flat.bboxes(4,:) - flat.bboxes(2,:);
names{end+1}   = 'inria 20080211 test';

load(grandCentralDir('test/1159407741750/annotations_20080221.mat'));
flat = flattenAnnotations(annotationsHier);
ymaxes{end+1}  = flat.bboxes(4,:);
heights{end+1} = flat.bboxes(4,:) - flat.bboxes(2,:);
names{end+1}   = 'grandcentral 1159407741750 test';

load(grandCentralDir('test/notregct02/annotations_20080221.mat'));
flat = flattenAnnotations(annotationsHier);
ymaxes{end+1}  = flat.bboxes(4,:);
heights{end+1} = flat.bboxes(4,:) - flat.bboxes(2,:);
names{end+1}   = 'grandcentral notregct02 test';

%% Heights Histograms

bins       = 0:max(cellfun(@max, heights));
binsZoomed = 0:256;
plotArgs       = cell(length(names)*2,1);
plotArgsZoomed = cell(length(names)*2,1);
for i=1:length(names)
  n = hist(heights{i}, bins);
  smoothed = imfilter(n, fspecial('gaussian',150,10), 0, 'same'); % for nicer-looking plots
  plotArgs{i*2-1} = bins;
  plotArgs{i*2}   = smoothed / sum(smoothed);

  n = hist(heights{i}, [binsZoomed binsZoomed(end)+eps]);
  n = n(1:end-1);
  plotArgsZoomed{i*2-1} = binsZoomed;
  plotArgsZoomed{i*2}   = n / sum(smoothed);
end

subplot(3,1, 1:2);
plot(plotArgs{:});
legend(names);
axis tight; a = axis;
line([50 50], [0 a(4)], 'LineStyle','-', 'Color',[1 1 1]*0.5, 'LineWidth',2);
line([64 64], [0 a(4)], 'LineStyle','-', 'Color',[1 1 1]*0.5, 'LineWidth',2);
line([128 128], [0 a(4)], 'LineStyle','-', 'Color','black', 'LineWidth',2);
set(gca, 'yticklabel',[]);
grid on;
title('distribution of pedestrian heights across several annotation sets');
ylabel('full distr (smoothed)');

subplot(313);
plot(plotArgsZoomed{:});
axis tight; a = axis;
line([50 50], [0 a(4)], 'LineStyle','-', 'Color',[1 1 1]*0.5, 'LineWidth',2);
line([64 64], [0 a(4)], 'LineStyle','-', 'Color',[1 1 1]*0.5, 'LineWidth',2);
line([128 128], [0 a(4)], 'LineStyle','-', 'Color','black', 'LineWidth',2);
set(gca, 'yticklabel',[]);
grid on;
xlabel('pixels');
ylabel('zoomed raw');

multisavefig(gcf, todaysResultsDir('heightsHist'), 'png;fig', [6 4], 100)

set(gcf,'position', [200 200 580 260]);

%% Heights vs Foot Y 

figure(1); clf;
figure(2); clf;

gcI = 1:length(names);%strmatch('grandcentral', names);
for j=1:length(gcI)
  i = gcI(j);
  
  figure(1); 
  subplot(1,length(gcI), i);
  set(gca,'fontsize',8);
  xx = ymaxes{i}(:);
  yy = heights{i}(:);
  ghist2([xx yy], [25 25]);
  xlabel('foot y');  ylabel('height');  title(names{i});
  title(split(' ',names{i}));

  figure(2); 
  subplot(1,length(gcI), i);
  set(gca,'fontsize',8);
  plot(xx,yy,'x');
  xlabel('foot y');  ylabel('height');  title(names{i});
  axis equal; 
  title(split(' ',names{i}));
end

figure(1); 
multisavefig(gcf, todaysResultsDir('heightsVsY_hist'), 'png;fig', [8 2], 300)
set(gcf,'position', [200 200 580 175]);

figure(2); 
multisavefig(gcf, todaysResultsDir('heightsVsY_scatter'), 'png;fig', [6 1.5], 100)
set(gcf,'position', [200 200 620 150]);

%% Clean up if run as a script

close all;