%% exptExploreProportions
% This script is an experiment to explore body proportions in the INRIA
% pedestrian dataset with Gerald's new annotations.

%% Store README info

F = fopen(todaysResultsDir('README.bbox.txt'), 'w');
fprintf(F,'bbox* created by %s\n', which(mfilename));
fclose(F);

%% Get data & compute various stats
load(inriaDir('test/inria/annotations_20080211.mat'));
flat = flattenAnnotations(annotationsHier);

headheights = 2*(flat.headcenters(2,:) - flat.bboxes(2,:));
boxheights  = (flat.bboxes(4,:) - flat.bboxes(2,:));
boxwidths   = (flat.bboxes(3,:) - flat.bboxes(1,:));

bwright = 2*(flat.bboxes(3,:) - flat.headcenters(1,:));
bwleft  = 2*(flat.headcenters(1,:) - flat.bboxes(1,:));
boxwidthsmin = min([bwright;bwleft]);

assert(all(headheights>0));
assert(all(boxheights>0));
assert(all(boxwidths>0));

wnorm = boxwidths./headheights;
hnorm = boxheights./headheights;

data = [wnorm;hnorm]';
[m,cv] = ransacGaussian(data);

meanHalfWidthAspect = mean((boxwidthsmin/2) ./ boxheights);

F = fopen(todaysResultsDir('bboxstats.txt'),'w');
fprintf(F, 'Mean bboxwidth/headheights = %f\n',  m(1));
fprintf(F, 'Mean bboxheight/headheights = %f\n', m(2));
fprintf(F, 'Mean min(left/right bboxwidth)/headheights = %f\n',  mean(boxwidthsmin./headheights));
fprintf(F, 'For a 128px tall person, \n');
fprintf(F, '  the mean min(half-width) should be %fpx\n', 128*meanHalfWidthAspect);
fprintf(F, '  and the head center should be %fpx below the top\n', 128/m(2)/2);
fprintf(F, '\n');
fclose(F);

%% Show distr of w&h normalized by head size

figure(1); clf;
ghist2([wnorm;hnorm]', [25 25]);
xlabel('width'); ylabel('height');
multisavefig(gcf, todaysResultsDir('bboxScatterDistr'), 'png;fig', [6 4], 100)

figure(2); clf;
plot(wnorm, hnorm, '.');
[v,d] = eigs(cv);
angle = atan2(v(2,1), v(1,1));
hold on;
a = axis;
ellipse(m, sqrt(diag(d))*1.0, 33, angle, 'r-','LineWidth',3);
for sigma=2:10
  ellipse(m, sqrt(diag(d))*sigma, 33, angle, 'r:');
end
axis(a);
grid on; xlabel('width'); ylabel('height');
multisavefig(gcf, todaysResultsDir('bboxHist2'), 'png;fig', [6 4], 100)


%% Show all those with default head sizes
if 0
  defaultHeadSize = 16*2;

  figure(3);
  for i=find(headheights==defaultHeadSize)
    subplot('Position',[0 0 1 1]);
    img = imread(inriaDir(fullfile('Train', flat.imagename{i})));
    imshow(img);
    drawAnnotation(flat.headcenters(:,i),flat.bboxes(:,i));
    set(gcf,'Units','pixels');
    p = get(gcf,'Position');
    set(gcf,'Position',[100 100 size(img,2) size(img,1)]);
    set(gcf,'Name',sprintf('%s (%d) : [w=%1.2f, h=%1.2f]', flat.imagename{i}, i, wnorm(i), hnorm(i)));
    pause;
  end
end

%% Show worst outliers 
% We show the poorest fits to our model to try to catch annotation mistakes.

lik = gaussLik(m,cv,data);
[yy,ii] = sort(lik);
% i = ii(1) % worst fit is a guy reclining on a very steep hill
% i = ii(2) % 2nd worst is a person with arms extended
% i = ii(3) % 3rd is another recliner

for i=ii(1:7)'
  figure(i);
  subplot('Position',[0 0 1 1]);
  img = imread(inriaDir(fullfile('test','inria', flat.imagename{i})));
  imshow(img);
  drawAnnotation(flat.headcenters(:,i),flat.bboxes(:,i));
  set(gcf,'Units','pixels');
  p = get(gcf,'Position');
  set(gcf,'Position',[100 100 size(img,2) size(img,1)]);
  set(gcf,'Name',sprintf('%s : [w=%1.2f, h=%1.2f]', flat.imagename{i}, wnorm(i), hnorm(i)));
  %pause;
end
