function extractNegativeExamples(basedir, annoset, nmaxname, w,fullh, fracTop, innerpadding,outerpadding, startscale,endscale,numPerImg,maxRetries, isnear)
%extractNegativeExamples(basedir, annoset, nmaxname, w,fullh, fracTop, innerpadding,outerpadding, startscale,endscale, numPerImg,maxRetries, isnear)
%   Takes a hierarchical annotations structure and writes out Dalal-sized
%   negative samples, using non-maximal suppression on the actual positive
%   set. 
%
%   Output will go to 
%     <basedir>/neg_<annoset>_<cropset>*
%   where <cropset> is of the form AAxBB_HCC, where AA is inner-padded
%   detector window width, BB is the inner-padded window height, and CC is
%   the fullh value.  The inner padding can be recovered by computing
%   (BB-CC)/2.  The outer padding can be recovered by taking 
%   (imageHeight-CC)/2.
%
%EXAMPLE:
%   doExtractNegativeExamples

h            = fullh * fracTop;
chipHeight = h + innerpadding*2 + outerpadding*2;
chipWidth  = w + innerpadding*2 + outerpadding*2;

cropset = sprintf('w%dx%dc8x8', w+innerpadding*2, h+innerpadding*2);

load(fullfile(basedir, ['annotations_' annoset '.mat']), 'annotationsHier');
anno = annotationsHier;
outreldir = ['neg_' annoset '_' cropset];
outdir    = fullfile(basedir, outreldir);
outlst    = fullfile(basedir, [outreldir '_' nmaxname '.lst']);

warning off; %#ok<WNOFF>
mkdir(outdir);%system(sprintf('C:/cygwin/bin/mkdir "%s"', strrep(outdir,'\','/'))); %
F  = fopen(outlst, 'w');

% foreach image
for iimg = 1:length(anno)
  % get data
  N = size(anno(iimg).bboxes,2);
  if ~N, continue; end
  img = imread(fullfile(basedir, anno(iimg).imagename));
  bboxesOrig = anno(iimg).bboxes; bboxesCrop = bboxesOrig;
  bboxesCrop(4,:) = bboxesOrig(2,:) + (bboxesOrig(4,:) - bboxesOrig(2,:)) * fracTop;
  paddedBboxes = bboxesCrop;
  paddedBboxes(1:2,:) = paddedBboxes(1:2,:) - innerpadding - outerpadding;
  paddedBboxes(3:4,:) = paddedBboxes(3:4,:) + innerpadding + outerpadding;
  [dummy, imgname] = fileparts(anno(iimg).imagename);

  if isnan(endscale)
    logEnd = min(log([...
      size(img,1)/chipHeight,...
      size(img,2)/chipWidth])); % autoselect
  else
    logEnd = log(endscale);
  end
  logStart = log(startscale);
  logDiff  = logEnd - logStart;
  
  numNeg  = 0;
  retries = 0;
  while numNeg<numPerImg && retries<maxRetries
    % sample the window
    scale = exp(rand(1)*logDiff + logStart);
    sampleWidth  = floor(chipWidth *scale);
    sampleHeight = floor(chipHeight*scale);
    maxXmin = size(img,2) - sampleWidth;
    maxYmin = size(img,1) - sampleHeight;
    xymin = rand(2,1).*[maxXmin;maxYmin] + 1;
    bbox = floor([xymin ; xymin+[sampleWidth;sampleHeight]]);
    
    % skip windows that would count as positive examples
    if bboxLabel(bbox, paddedBboxes, isnear)
      retries = retries + 1;
      continue;
    end
    
    % extract window
    cropped = imresize(imcrop(img, [bbox(1:2), bbox(3:4)-bbox(1:2)]), [chipHeight chipWidth], 'lanczos3');
    
    % output image & index
    fname = sprintf('%s_%03dx%03d-%03dx%03d.png', imgname, bbox);
    imwriteIfNeeded(cropped, outdir, fname);
    fprintf(F, '%s/%s\n', outreldir, fname);
    
    numNeg = numNeg + 1;
  end
end
fclose(F);

%-------------------------------
function imwriteIfNeeded(im,d,fname)
fname = fullfile(d,fname);
%if ~exist(fname, 'file')
  imwrite(im,fname);
%end

