function sampleFramesFromOneVideo(infile, outdir, N, deint, numFrames)
%sampleFramesFromOneVideo(infile, outdir, N, deint, numFrames)
%  Samples <N> frames from <INFILE> and places the frames in
%  <OUTDIR>/%07d.png.  If <deint> is true, the frames are
%  deinterlaced using a temporal median filter for 0-indexed odd lines.
%  Assumes the video has <numFrames> frames in it.  If <numFrames> is not
%  specified, we try to auto-detect the video length.
%
%  Frame numbers are 1-indexed.

if nargin<4
  deint=0;
end

mkdir(outdir);
%delete(fullfile(outdir, '*'));

if ispc
  vr = videoReader(infile, 'frameTimeoutMS',-1);
else
  vr = videoReader(infile, 'ffmpegDirect');
end
info = get(vr) %#ok<NOPRT>
if (nargin<5)
  if info.numFrames > 0
    numFrames = info.numFrames;
  else 
    numFrames = 60*30*60; % default: 1 hour @ 30fps
  end
end

frameNums = floor((numFrames-1)*sort(rand(1,N)) + 1);
for f=1:max(frameNums) % frameNums%
  assert(logical(seek(vr,f-1)));
  outfile = [outdir '/' sprintf('%07d.png', f)];
  if (ismember(f, frameNums))
    frame = getframe(vr);
    if deint
      % temporal median filter deinterlacing, adapted from the description:
      % http://www.mathworks.com/access/helpdesk/help/toolbox/vipblks/index.html?/access/helpdesk/help/toolbox/vipblks/ref/deinterlacing.html
      for odd=2:2:size(frame,1)-1
        frame(odd,:,:) = median(frame(odd-1:odd+1,:,:), 1);
      end
    elseif all(size(frame) == [1080, 1440, 3])
      frame = imresize(frame, [1080, 1920], 'nearest');
    elseif all(size(frame) == [1440, 1080, 3])
      frame = imresize(frame, [1920, 1080], 'nearest');
    end
    imwrite(frame, outfile);
    fprintf('wrote %s\n', outfile);
  end
end

vr = close(vr); %#ok<NASGU>