function linuxPath = linuxPath(varargin)
%linuxPath = linuxPath(localPath)
%  Performs virtual path translation for APP network drives.  Supply a
%  path in the local filesystem that refers to data on an NFS/CIFS 
%  directory, and this function attempts to translate it to a linux path. 
%  On Linux, LINUXPATH=LOCALPATH.  Relative paths pass untranslated on all
%  platforms.  
%
%  The available mappings are stored in appMappings.csv.  They should be
%  self-explanatory for now.
%  

if nargin>1
  localPath = fullfile(varargin{:});
else
  localPath = varargin{1};
end

% do nothing on non-Windows
if (~ispc), linuxPath = localPath; return; end
% do nothing on relative paths
if isempty(ismember(localPath(1),'/\')), linuxPath = localPath; return; end

localPath = strrep(localPath, '\', '/');

mappings = getMappings;
for i=1:size(mappings,1)
  if doSubs(mappings{i,:})
    return;
  end
end
error('Unknown path'); 

%-------------------------------
  function done = doSubs(prefix, linuxPrefix)
    nL = numel(localPath);
    nP = numel(prefix);
    if nL < nP
      done = 0;
    elseif ~strcmp(localPath(1:nP), prefix)
      done = 0;
    elseif localPath(nP+1)~='/'
      done = 0;
    else
      done = 1;
      linuxPath = [strrep(linuxPrefix,'\','/') '/' strrep(localPath(nP+2:end),'\','/')];
    end
  end

%-------------------------------
  function mappings = getMappings
    % gets the reverse mappings
    persistent mappings_;
    if isempty(mappings_)
      % read mappings file
      d = fileparts(which(mfilename));
      mappingsFile = fullfile(d, 'appMappings.csv');
      F=fopen(mappingsFile);
      txt = fread(F, '*char')';
      fclose(F);
      
      % break into lines
      lines = split(sprintf('\r\n'), txt);
      
      % first line gives a machine name list...find the current box
      tokens = split(',', lines{1});
      assert(strcmp(tokens{1},'linux'));
      [tf,col] = ismember(lower(getenv('COMPUTERNAME')), tokens);
      assert(tf, 'Could not find mappings for "%s" in "%s"',...
        lower(getenv('COMPUTERNAME')), mappingsFile);
      
      % extract the mappings that exist for this box
      mappings_ = cell(0,2);
      for j=2:length(lines)
        tokens = rsplit(',', lines{j});
        if length(tokens)>=col && ~isempty(tokens{col})
          mappings_{end+1,1} = tokens{col}; %#ok<AGROW>
          mappings_{end,2}   = tokens{1};
        end
      end
    end
    mappings = mappings_;
  end
end
