function out = map(f, in, varargin)
%OUT = MAP(FUNC, IN, ...)
%  Applies function FUNC to each element of IN and assembles a return
%  value from the results.
%
%  If IN is an array, OUT will be an array of the same dimensions.
%  This function acts as a wrapper to ARRAYFUN.
%
%  If IN is a cell array, OUT will be a cell

if ~isempty(varargin)
  classEq = @(x) strcmp(class(x),class(in));
  assert(all(cellfun(classEq, varargin)), ...
    'All container objects must be of the same type');
end

if isstruct(in)
  out = structfun(f, in, varargin{:}, 'UniformOutput',0);
elseif iscell(in)
  out = cellfun(f, in, varargin{:}, 'UniformOutput',0);
else
  out = arrayfun(f, in, varargin{:});
end
