function img = imtile(images,varargin)
%montage = imtile(images)
%  Creates a montage by tiling the images...TODO
%
%tile(...,paramName,paramValue)
%  Adjusts the default parameter settings.
%
%  Parameter Default Description
%  --------- ------- -----------
%  TODO: fill in params here
%
%montage = imtile({ imread('board.tif'), {rand(25), []} });
%
%TODO: esp. if I do the autoscaling thing, allow embedding of arbitrary
%frame-specific modifiers, e.g.
%  montage = imtile({ imread('board.tif'), {rand(25), [], 'ResizeMethod','bicubic'} });
%
%TODO: make it possible for the number of pixels between to cells to be an
%odd number.
%
%TODO: if nargout==0, do imshow on the result
%
%TODO:
%  * take a stack of images (plus [nrows,ncols]?)
%  * take a 1D cell array plus [numrows,ncols]
%  * take a hierarchical structure of cell arrays?
%  * allow letterboxing vs assymetric rescaling
%    -- allow left/center/right, top/middle/bottom alignment
%  * default args after cell array, overridden by cell entries

args = parseArguments(varargin);

% number of tiles
TR = size(images,1);
TC = size(images,2);

% size of each tile
tileWidths = zeros(1,TC);
tileHeights = zeros(1,TR);
for tr=1:TR
  for tc=1:TC
    tileWidths(tc) = max(tileWidths(tc), size(getimg(tr,tc), 2));
    tileHeights(tr) = max(tileHeights(tr), size(getimg(tr,tc), 1));
  end
end

% create the output image with borders
img = zeros(...
  args.OuterBorderHeight*2 + sum(tileHeights+2*args.BorderHeight),...
  args.OuterBorderWidth*2 + sum(tileWidths+2*args.BorderWidth), 'uint8');
img = cat(3, ...
  img+uint8(255*args.BorderColor(1)), ...
  img+uint8(255*args.BorderColor(2)), ...
  img+uint8(255*args.BorderColor(3)));

% fill in tiles
yStart = args.OuterBorderHeight + args.BorderHeight+1;
for tr=1:TR
  yEnd = yStart + tileHeights(tr) - 1;
  xStart = args.OuterBorderWidth + args.BorderWidth+1;
  for tc=1:TC
    xEnd = xStart + tileWidths(tc) - 1;

    if isempty(getimg(tr,tc))
      for channel=1:3
        img(yStart:yEnd, xStart:xEnd, channel) = uint8(255*args.BackgroundColor(channel));
      end
    else
      % xy resize
      preResize = getimg(tr,tc);
      if (size(preResize,1) == tileHeights(tr)) && (size(preResize,2) == tileWidths(tc))
        rz = preResize;
      else 
        rz = imresize(preResize, [tileHeights(tr) tileWidths(tc)], ...
          args.ResizeMethod, ...
          'Antialiasing', args.Antialiasing);
      end
      
      % color/intensity scaling
      [clims,doColorScaling] = getClims(tr,tc);
      if doColorScaling && ~isempty(rz)
        rz = double(rz);
        for channel = 1:size(rz,3)
          rz(:,:,channel) = rz(:,:,channel) / double(diff(clims)) + clims(1);
        end
      end

      % gray -> rgb
      if ndims(rz) == 2
        rz = cat(3, rz,rz,rz);
      elseif ndims(rz) == 3
        % nothing special here...
      else
        error('only grayscale and RGB images are supported right now');
      end

      % embed data
      img(yStart:yEnd, xStart:xEnd, :) = uint8(rz);
    end

    xStart = xEnd+1 + 2*args.BorderWidth;
  end
  yStart = yEnd+1 + 2*args.BorderHeight;
end

%-------------------------------------------------------------------------
  function args = parseArguments(vargs)

    aliases = cell(0,2);

    args = struct;

    args.BorderWidth  = 2;
    args.BorderHeight = 2;
    aliases = cat(1, aliases, {'Border', {'BorderWidth','BorderHeight'}});

    args.NOuterBorders = 1;

    args.BorderColor     = [1 1 1];
    args.BackgroundColor = [1 1 1];
    aliases = cat(1, aliases, {'Color', {'BorderColor', 'BackgroundColor'}});

    args.Antialiasing = 1;
    args.ResizeMethod = 'bicubic';
    % TODO: a way to control scaling
    % TODO: auto convert to grayscale
    % TODO: double vs. uint8
    % TODO: scaling

    %-----------------

    args = parseArgs(vargs,args,[],aliases);
    % TODO: validate args

    args.OuterBorderWidth  = args.NOuterBorders * args.BorderWidth;
    args.OuterBorderHeight = args.NOuterBorders * args.BorderHeight;

  end

%-------------------------------------------------------------------------
  function ii = getimg(tr,tc)

    ii = images{tr,tc};
    if iscell(ii)
      ii = ii{1};
    end

  end

%-------------------------------------------------------------------------
  function [clims,doColorScaling] = getClims(tr,tc)

    ii = images{tr,tc};
    if iscell(ii) && numel(ii) == 1
      ii = ii{:};
    end
    if iscell(ii) 
      clims = ii{2};
      ii    = ii{1};
      
      if isempty(clims)
        clims = [min(ii(:)) max(ii(:))];
      end
    else
      if isa(ii, 'uint8')
        clims = [0 255];
      elseif isa(ii, 'uint16')
        clims = [0 65535];
      else
        clims = [0 1];
      end
    end

    % We'll be converting everything to uint8, so it's easiest to embed the
    % scaling factor here.
    % TODO: make sure the scaling stuff actually works for non-zero clims(1). 
    clims = clims / 255;
    
    doColorScaling = 1;
    if isa(ii, 'uint8') && all(clims == [0 255])
      doColorScaling = 0;
    end

  end

end
