function p = resamplingTestPattern
%p=resamplingTestPattern
%  Returns an image containing black and white pixels that's useful for
%  testing image resampling algorithms.
%
%resamplingTestPattern
%  Saves the image to to a canonical location for usage with the
%  demoimresize application.
%

pattSz = 64;
blank = zeros(pattSz, pattSz, 'uint8');
xy = (-pattSz/2+0.5:pattSz/2-0.5);
[x,y] = meshgrid(xy,xy);
L_75  = (abs(x).^0.75 + abs(y).^0.75).^(1/0.75);
L_85  = (abs(x).^0.85 + abs(y).^0.85).^(1/0.85);
L1    = abs(x) + abs(y);
L1_25 = (abs(x).^1.25 + abs(y).^1.25).^(1/1.25);
L1_50 = (abs(x).^1.5 + abs(y).^1.5).^(1/1.5);
L2    = sqrt(x.^2 + y.^2);
L3    = (abs(x).^3 + abs(y).^3).^(1/3);
Linf  = max(abs(x),abs(y));

measures = {L_75,L_85,L1,L1_25,L1_50,L2,L3,Linf};
spacings = 1:5;
subpatterns = cell(length(spacings), length(measures));
for mi = 1:length(measures)
  m = measures{mi};
  for si = 1:length(spacings)
    s = spacings(si);
    subpatterns{si,mi} = pattern(m,s);
  end
end

p = imtile(subpatterns, 'BorderWidth',0, 'BorderHeight',0);

if nargout == 0
  imwrite(p, '../../learcode/test/resamplingTestPattern.png');
  imshow(p);
end

  function p = pattern(measure,spacing)
    p = blank;
    p(mod(floor(measure/spacing), 2) == 0) = 255;
  end
end