%% Create HDP Visualizations in Batch
% Ideally, we'd want the visualizations to be produced with
% ../experiments/makefile.  Unfortunately, it's most robust to use the
% -nodisplay feature with matlab when doing mass executions (to avoid
% killing off Matlab when we lose the X connection).  When -nodisplay is
% used, the PRINT function causes Matlab to crash (it doesn't even throw an
% exception).  So, we have this script that looks for finished HDP model
% files and does the visualization here.

ww = warning;
warning off; %#ok<WNOFF>
addNeededPaths;

for annoset = {'20080211', 'orig'}
  pp = appPath(['/scratch/dalleyg/dalalTriggs/trunk/experiments/inria_' annoset{1}]);
  pp(pp=='\') = '/';
  fprintf('\nSEARCHING FOR COMPLETED MODELS IN %s\n', pp);
  [st,p] = system(['ls -t ' pp '/*/persp/*/*/*/model.mat']);
  
  if st==0
    p = sort(split([10 13], p));
    p = cellfilt(@length, p); % strip last blank line if present
    fprintf('  (%d POTENTIALLY FOUND)\n',length(p));
    for i=1:length(p)
      dd = dir(p{i});
      if dd.bytes > 0
        fprintf('[%s] %s ...  ', dd.date, p{i});
        outdir = p{i}(1:find(ismember(p{i},'/\'),1,'last'));
        if exist(fullfile(outdir,'topicEntropyStats.txt'), 'file') %floorplan-topic0001
          fprintf('  already done.\n');
        else
          visClusteringResults(p{i});
        end
      end
    end
  end
end

fprintf('\nDONE.\n');

warning(ww);
