function h = prPlot(results, thistitle,legtext,axislimit,fontsize, styleOffs)
%h = prPlot(results, thistitle,legtext,axislimit,fontsize, styleOffs)
%  results: structs of cell array of arrays with the following fields:
%              achievableRecallSparse
%              achievablePrecisionSparse
%              achievableRecallDense
%              achievablePrecisionDense
%           elements are accessed as 
%              results.(fieldname){curveIdx}(exptIdx)
%

[marker,style,color] = getStyles;

N = length(results.achievableRecallSparse);

if ~exist('thistitle','var') || isempty(thistitle) 
  thistitle = '';
end
if ~exist('legtext','var') || isempty(legtext) 
  legtext = {};
end
if ~exist('axislimit','var') || isempty(axislimit) 
  axislimit = [0 1 0 1];
end
if ~exist('fontsize','var') || isempty(fontsize) 
  fontsize = 8;
end
if ~exist('styleOffs','var') || isempty(styleOffs) 
  styleOffs = 0:N-1;
end

if numel(styleOffs) == 1 && N > 1
  styleOffs = styleOffs + 0:N-1;
end

%%%%%%%%%%%%%%%%%%%% NOW DRAW PR curves
cla reset;

hold on;
% draw in 3 passes so that the legend works
for pass=1:3
  for i=1:N
    colorindex  = mod(styleOffs(i),length(color))+1;
    styleindex  = mod(styleOffs(i),length(style))+1;
    markerindex = mod(styleOffs(i),length(marker))+1;
    
    if pass == 1
      % establish plot styles
      line(results.achievableRecallSparse{i}(2),...
        results.achievablePrecisionSparse{i}(2),...
        'Color',color(colorindex,:),...
        'LineStyle',style{styleindex},'LineWidth',2,...
        'Marker',marker(markerindex), 'MarkerSize',8);
    elseif pass == 2
      % draw dense curve
      line(results.achievableRecallDense{i},...
        results.achievablePrecisionDense{i},...
        'Color',color(colorindex,:),...
        'LineStyle',style{styleindex},'LineWidth',2);
    else
      % include a sparse set of markers
      line(results.achievableRecallSparse{i},...
        results.achievablePrecisionSparse{i},...
        'Color',color(colorindex,:),...
        'LineStyle','none',...
        'Marker',marker(markerindex), 'MarkerSize',8);
    end
  end
end
hold off;

axisfont=fontsize;
labelfont=fontsize;
set(gca,'Box','on');
set(gca,'FontSize',axisfont);

titletext=(['Precision-Recall Curves - ',thistitle]);
title(titletext,'FontSize',labelfont);grid on;
xlabel('recall','FontSize',labelfont); grid on;
ylabel('precision','FontSize',labelfont); grid on;
                    
if (~isempty(legtext))
  hleg=legend(legtext{:},'location','southwest');
  set(hleg, 'interpreter','none', 'FontSize',labelfont);
end

axis(axislimit);

set(gca,'XMinorGrid','off');
set(gca,'YMinorGrid','off');
%%%%%%%%%%%%%%%%% Return figure handle now
h = gcf;

%plotedit on; % set plot editable
