#!/usr/bin/perl
use strict;

print "This script is used to create a local cached copy of all of the \n".
      "training and test data and put it in our preferred directory \n".
      "structure.  It will take a long time when run the first time \n".
      "(especially if your internet connection is slow).  For non-CSAIL \n".
      "machines, about 1GB is downloaded the first time this is run.  For \n".
      "CSAIL machines, much more data is downloaded.";

`(rm sourcedata 2>&1) > /dev/null`;
`(mkdir sourcedata 2>&1) > /dev/null`;

my $csailsourcedata = '/data/app/images/pedestrianDetection'; 
if (-f $csailsourcedata) {
  # If on the MIT CSAIL network, make a local cached copy of all our datasets,
  # including the INRIAPerson one as well as internal-only ones.
  die "please install rsync\n" unless (run("which rsync"));
  run("rsync -a --delete $csailsourcedata/* sourcedata")
  
} else {
  # Just get the original INRIA dataset...the Grand Central dataset is not
  # public.
  die "please install wget\n" unless (run("which wget"));
  run("wget http://pascal.inrialpes.fr/data/human/INRIAPerson.tar") unless (-f "INRIAPerson.tar");
  run("cd sourcedata && tar -xf ../INRIAPerson.tar") unless (-d "sourcedata/INRIAPerson");
  # Convert from Dalal's directory structure to ours.  Ours makes it easier
  # to combine multiple datasets with our batch execution scripts.
  if (!(-f "sourcedata/INRIAPerson/train/inria/pos_orig_w64x128c8x8_all.lst")) {
    chdir "sourcedata/INRIAPerson";
    
    # Rearrange training data
    run("mv Train Train.tmp") if (-d "Train/neg");
    run("mkdir train       2>&1 > /dev/null", 0);
    run("mkdir train/inria 2>&1 > /dev/null", 0);
    chdir "train/inria";
    run("mv ../../Train.tmp/annotations     annotations_orig     2>&1 > /dev/null", 0);
    run("mv ../../Train.tmp/annotations.lst annotations_orig.lst 2>&1 > /dev/null", 0);
    run("mv ../../Train.tmp/pos             pos_full             2>&1 > /dev/null", 0);
    run("mv ../../Train.tmp/pos.lst         pos_full.lst         2>&1 > /dev/null", 0);
    run("mv ../../Train.tmp/neg             neg_full             2>&1 > /dev/null", 0);
    run("mv ../../Train.tmp/neg.lst         neg_full.lst         2>&1 > /dev/null", 0);
    run("rmdir ../../Train.tmp") unless (!(-d "../../Train.tmp"));
    run("rm -rf ../../train_64x128_H96/pos     2>&1 > /dev/null", 0); # it's a bunch of symlinks
    run("mv ../../train_64x128_H96/pos.lst  pos_orig_w64x128c8x8_all.lst  2>&1 > /dev/null", 0);
    run("rm ../../train_64x128_H96/neg     2>&1 > /dev/null", 0); # it's a symlink
    run("rm ../../train_64x128_H96/neg.lst 2>&1 > /dev/null", 0); # it's redundant
    run("rm -rf ../../train_64x128_H96") unless (!(-d "../../train_64x128_H96"));
    run("mv ../../96X160H96/Train/pos pos_orig_w64x128c8x8  2>&1 > /dev/null", 0);
    run("rmdir ../../96X160H96/Train") unless (!(-d "../../96X160H96/Train"));
    run("rmdir ../../96X160H96")       unless (!(-d "../../96X160H96"));
    chdir "../..";
   
    # Rearrange test data
    run("mv Test Test.tmp") if (-d "Test/neg");
    run("mkdir test       2>&1 > /dev/null", 0);
    run("mkdir test/inria 2>&1 > /dev/null", 0);
    chdir "test/inria";
    run("mv ../../Test.tmp/annotations     annotations_orig     2>&1 > /dev/null", 0);
    run("mv ../../Test.tmp/annotations.lst annotations_orig.lst 2>&1 > /dev/null", 0);
    run("mv ../../Test.tmp/pos             pos_full             2>&1 > /dev/null", 0);
    run("mv ../../Test.tmp/pos.lst         pos_full.lst         2>&1 > /dev/null", 0);
    run("mv ../../Test.tmp/neg             neg_full             2>&1 > /dev/null", 0);
    run("mv ../../Test.tmp/neg.lst         neg_full.lst         2>&1 > /dev/null", 0);
    run("rmdir ../../Test.tmp") unless (!(-d "../../Test.tmp"));
    run("rm -rf ../../test_64x128_H96/pos     2>&1 > /dev/null", 0); # it's a bunch of symlinks
    run("mv ../../test_64x128_H96/pos.lst  pos_orig_w64x128c8x8_all.lst  2>&1 > /dev/null", 0);
    run("rm ../../test_64x128_H96/neg     2>&1 > /dev/null", 0); # it's a symlink
    run("rm ../../test_64x128_H96/neg.lst 2>&1 > /dev/null", 0); # it's redundant
    run("rm -rf ../../test_64x128_H96") unless (!(-d "../../test_64x128_H96"));
    run("mv ../../70X134H96/Test/pos pos_orig_w64x128c8x8  2>&1 > /dev/null", 0);
    run("rmdir ../../70X134H96/Test") unless (!(-d "../../70X134H96/Test"));
    run("rmdir ../../70X134H96")       unless (!(-d "../../70X134H96"));
    chdir "../..";
  
    # Dalal used paths relative to the current execution directory for his 
    # .lst files (or something like that).  We've changed the code to use paths
    # relative to the .lst file.  We need to patch up all the .lst files as 
    # appropriate.
    patchLst("train/inria/neg_full.lst",                 "neg_full");
    patchLst("train/inria/pos_full.lst",                 "pos_full");
    patchLst("train/inria/pos_orig_w64x128c8x8_all.lst", "pos_orig_w64x128c8x8_all");
    patchLst("test/inria/neg_full.lst",                  "neg_full");
    patchLst("test/inria/pos_full.lst",                  "pos_full");
    patchLst("test/inria/pos_orig_w64x128c8x8_all.lst",  "pos_orig_w64x128c8x8_all");
    
    chdir "../..";
  }
}

#############################
sub run($$) {
  my ($cmd, $checkst) = @_;
  $checkst = 1 if (!defined $checkst);
  open F, "$cmd | ";
  my $results;
  while (<F>) {
    $results .= $_;
  }
  close F;
  if ($checkst) {
    die "$cmd exited with error code $?" if ($? != 0);
  }
  return $results;
}

sub patchLst($$) {
  my ($lst, $dir) = @_;
  my $out;
  open F, $lst;
  while (<F>) {
    my @parts = split '/';
    $out .= "$dir/$parts[$#parts]";
  }
  close F;
  open F, ">$lst";
  print F $out;
  close F;
}
