#!/usr/bin/perl
use POSIX qw(strftime);
use strict;

# Script for creating nicely formatted subversion tags.

(@ARGV == 1) || die "Tag name needed as a command-line argument.";
my $tagname = $ARGV[0];
($tagname =~ /[0-9a-zA-Z_\-\.\+\=]+/) || die "Tagname must only be alphanumeric + [_-=+.]\n";

my $today = strftime "%Y/%m/%d", localtime(time);
my $now   = strftime "%H%M", localtime(time);

# It's dangerous to do a tag without committing first.  Do it now, before
# we try to figure out the revision number, to be safe.  This is a no-op if
# no commits are needed.
`svn commit -m 'pre-commit for $today/$now-UNKNOWN-$tagname'`;
`svn up`;

my ($rev, $url);
foreach my $line (`svn info`) {
  if ($line =~ /revision:*\s+([0-9]+)/i) {
    $rev = $1;
  } elsif ($line =~ /URL:\s*([^\s]+)/i) {
    $url = $1;
  }
}
defined $rev || die "Could not find revision number ($rev).";
defined $url || die "Could not find the svn url.";
my @parts = split('/', $url);
my $parenturl = join('/', @parts[0..($#parts-1)]);

my $tag = "$today/$now-r$rev-$tagname";

#print "$tag\n";
#print "$url\n";
#print "$parenturl\n";

my $cmd = "svn mkdir $parenturl/tags/$today -m 'TAG: $tag' --parents 2>&1";
`$cmd`; # || die "\nCould not create directory for $tagname while executing:\n$cmd\n";

my $cmd = "svn cp $url $parenturl/tags/$tag -m 'TAG: $tag'";
`$cmd` || die "\nCould not create tag $tagname while executing:\n$cmd\n";
